@@ -14,6 +14,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import binascii
 import logging
 
 from django.core.urlresolvers import reverse
@@ -78,7 +79,7 @@ class ClustersTests(test.TestCase):
                                           'flavor_list')})
     def test_index_pagination(self):
         clusters = self.trove_clusters.list()
-        last_record = clusters[0]
+        last_record = clusters[1]
         clusters = common.Paginated(clusters, next_marker="foo")
         trove_api.trove.cluster_list(IsA(http.HttpRequest), marker=None)\
             .AndReturn(clusters)
@@ -130,54 +131,60 @@ class ClustersTests(test.TestCase):
 
     def test_launch_cluster_mongo_fields(self):
         datastore = 'mongodb'
-        fields = self.launch_cluster_fields_setup(datastore, '2.6')
+        datastore_version = '2.6'
+        fields = self.launch_cluster_fields_setup(datastore,
+                                                  datastore_version)
+        field_name = self._build_flavor_widget_name(datastore,
+                                                    datastore_version)
 
         self.assertTrue(self._contains_datastore_in_attribute(
-            fields['flavor'], datastore))
+            fields[field_name], field_name))
         self.assertTrue(self._contains_datastore_in_attribute(
-            fields['num_instances'], datastore))
+            fields['num_instances'], field_name))
         self.assertTrue(self._contains_datastore_in_attribute(
-            fields['num_shards'], datastore))
+            fields['num_shards'], field_name))
         self.assertFalse(self._contains_datastore_in_attribute(
-            fields['root_password'], datastore))
+            fields['root_password'], field_name))
         self.assertFalse(self._contains_datastore_in_attribute(
-            fields['num_instances_vertica'], datastore))
-        self.assertFalse(self._contains_datastore_in_attribute(
-            fields['vertica_flavor'], datastore))
+            fields['num_instances_vertica'], field_name))
 
     def test_launch_cluster_redis_fields(self):
         datastore = 'redis'
-        fields = self.launch_cluster_fields_setup(datastore, '3.0')
+        datastore_version = '3.0'
+        fields = self.launch_cluster_fields_setup(datastore,
+                                                  datastore_version)
+        field_name = self._build_flavor_widget_name(datastore,
+                                                    datastore_version)
 
         self.assertTrue(self._contains_datastore_in_attribute(
-            fields['flavor'], datastore))
+            fields[field_name], field_name))
         self.assertTrue(self._contains_datastore_in_attribute(
-            fields['num_instances'], datastore))
+            fields['num_instances'], field_name))
         self.assertFalse(self._contains_datastore_in_attribute(
-            fields['num_shards'], datastore))
+            fields['num_shards'], field_name))
         self.assertFalse(self._contains_datastore_in_attribute(
-            fields['root_password'], datastore))
+            fields['root_password'], field_name))
         self.assertFalse(self._contains_datastore_in_attribute(
-            fields['num_instances_vertica'], datastore))
-        self.assertFalse(self._contains_datastore_in_attribute(
-            fields['vertica_flavor'], datastore))
+            fields['num_instances_vertica'], field_name))
 
     def test_launch_cluster_vertica_fields(self):
         datastore = 'vertica'
-        fields = self.launch_cluster_fields_setup(datastore, '7.1')
+        datastore_version = '7.1'
+        fields = self.launch_cluster_fields_setup(datastore,
+                                                  datastore_version)
+        field_name = self._build_flavor_widget_name(datastore,
+                                                    datastore_version)
 
-        self.assertFalse(self._contains_datastore_in_attribute(
-            fields['flavor'], datastore))
-        self.assertFalse(self._contains_datastore_in_attribute(
-            fields['num_instances'], datastore))
-        self.assertFalse(self._contains_datastore_in_attribute(
-            fields['num_shards'], datastore))
         self.assertTrue(self._contains_datastore_in_attribute(
-            fields['root_password'], datastore))
+            fields[field_name], field_name))
+        self.assertFalse(self._contains_datastore_in_attribute(
+            fields['num_instances'], field_name))
+        self.assertFalse(self._contains_datastore_in_attribute(
+            fields['num_shards'], field_name))
         self.assertTrue(self._contains_datastore_in_attribute(
-            fields['num_instances_vertica'], datastore))
+            fields['root_password'], field_name))
         self.assertTrue(self._contains_datastore_in_attribute(
-            fields['vertica_flavor'], datastore))
+            fields['num_instances_vertica'], field_name))
 
     @test.create_stubs({trove_api.trove: ('datastore_flavors',
                                           'datastore_list',
@@ -235,18 +242,19 @@ class ClustersTests(test.TestCase):
             datastore=cluster_datastore,
             datastore_version=cluster_datastore_version,
             nics=cluster_network,
-            root_password=None).AndReturn(self.trove_clusters.first())
+            root_password=None,
+            locality=None).AndReturn(self.trove_clusters.first())
 
+        field_name = self._build_flavor_widget_name(cluster_datastore,
+                                                    cluster_datastore_version)
         self.mox.ReplayAll()
         post = {
             'name': cluster_name,
             'volume': cluster_volume,
             'num_instances': cluster_instances,
             'num_shards': 1,
-            'num_instances_per_shards': cluster_instances,
-            'datastore': cluster_datastore + u'-' + cluster_datastore_version,
-            'flavor': cluster_flavor,
-            'network': cluster_network
+            'datastore': field_name,
+            field_name: 'aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa',
         }
 
         res = self.client.post(LAUNCH_URL, post)
@@ -291,18 +299,20 @@ class ClustersTests(test.TestCase):
             datastore=cluster_datastore,
             datastore_version=cluster_datastore_version,
             nics=cluster_network,
-            root_password=None).AndReturn(self.trove_clusters.first())
+            root_password=None,
+            locality=None).AndReturn(self.trove_clusters.first())
 
+        field_name = self._build_flavor_widget_name(cluster_datastore,
+                                                    cluster_datastore_version)
         self.mox.ReplayAll()
         post = {
             'name': cluster_name,
             'volume': cluster_volume,
             'num_instances': cluster_instances,
             'num_shards': 1,
-            'num_instances_per_shards': cluster_instances,
-            'datastore': cluster_datastore + u'-' + cluster_datastore_version,
-            'flavor': cluster_flavor,
-            'network': cluster_network
+            'datastore': field_name,
+            field_name: 'aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa',
+            'network': cluster_network,
         }
 
         res = self.client.post(LAUNCH_URL, post)
@@ -344,18 +354,19 @@ class ClustersTests(test.TestCase):
             datastore=cluster_datastore,
             datastore_version=cluster_datastore_version,
             nics=cluster_network,
-            root_password=None).AndReturn(self.trove_clusters.first())
+            root_password=None,
+            locality=None).AndReturn(self.trove_clusters.first())
 
+        field_name = self._build_flavor_widget_name(cluster_datastore,
+                                                    cluster_datastore_version)
         self.mox.ReplayAll()
         post = {
             'name': cluster_name,
             'volume': cluster_volume,
             'num_instances': cluster_instances,
             'num_shards': 1,
-            'num_instances_per_shards': cluster_instances,
-            'datastore': cluster_datastore + u'-' + cluster_datastore_version,
-            'flavor': cluster_flavor,
-            'network': cluster_network
+            'datastore': field_name,
+            field_name: 'aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa',
         }
 
         res = self.client.post(LAUNCH_URL, post)
@@ -380,6 +391,45 @@ class ClustersTests(test.TestCase):
         self.assertTemplateUsed(res, 'horizon/common/_detail.html')
         self.assertContains(res, cluster.ip[0])
 
+    @test.create_stubs({trove_api.trove: ('cluster_get',
+                                          'instance_get',
+                                          'flavor_get',)})
+    def test_details_without_locality(self):
+        cluster = self.trove_clusters.list()[1]
+        trove_api.trove.cluster_get(IsA(http.HttpRequest), cluster.id) \
+            .MultipleTimes().AndReturn(cluster)
+        trove_api.trove.instance_get(IsA(http.HttpRequest), IsA(str)) \
+            .MultipleTimes().AndReturn(self.databases.first())
+        trove_api.trove.flavor_get(IsA(http.HttpRequest), IsA(str)) \
+            .MultipleTimes().AndReturn(self.flavors.first())
+        self.mox.ReplayAll()
+
+        details_url = reverse('horizon:project:database_clusters:detail',
+                              args=[cluster.id])
+        res = self.client.get(details_url)
+        self.assertTemplateUsed(res, 'horizon/common/_detail.html')
+        self.assertNotContains(res, "Locality")
+
+    @test.create_stubs({trove_api.trove: ('cluster_get',
+                                          'instance_get',
+                                          'flavor_get',)})
+    def test_details_with_locality(self):
+        cluster = self.trove_clusters.first()
+        trove_api.trove.cluster_get(IsA(http.HttpRequest), cluster.id) \
+            .MultipleTimes().AndReturn(cluster)
+        trove_api.trove.instance_get(IsA(http.HttpRequest), IsA(str)) \
+            .MultipleTimes().AndReturn(self.databases.first())
+        trove_api.trove.flavor_get(IsA(http.HttpRequest), IsA(str)) \
+            .MultipleTimes().AndReturn(self.flavors.first())
+        self.mox.ReplayAll()
+
+        details_url = reverse('horizon:project:database_clusters:detail',
+                              args=[cluster.id])
+        res = self.client.get(details_url)
+        self.assertTemplateUsed(res, 'project/database_clusters/'
+                                     '_detail_overview.html')
+        self.assertContains(res, "Locality")
+
     @test.create_stubs(
         {trove_api.trove: ('cluster_get',
                            'cluster_grow'),
@@ -395,13 +445,15 @@ class ClustersTests(test.TestCase):
         instances = [
             cluster_manager.ClusterInstance("id1", "name1", cluster_flavor,
                                             cluster_flavor_name,
-                                            cluster_volume, "master", None),
+                                            cluster_volume, "master", None,
+                                            None),
             cluster_manager.ClusterInstance("id2", "name2", cluster_flavor,
                                             cluster_flavor_name,
-                                            cluster_volume, "slave", "master"),
+                                            cluster_volume, "slave",
+                                            "master", None),
             cluster_manager.ClusterInstance("id3", None, cluster_flavor,
                                             cluster_flavor_name,
-                                            cluster_volume, None, None),
+                                            cluster_volume, None, None, None),
         ]
 
         manager = cluster_manager.ClusterInstanceManager(cluster.id)
@@ -468,13 +520,15 @@ class ClustersTests(test.TestCase):
         instances = [
             cluster_manager.ClusterInstance("id1", "name1", cluster_flavor,
                                             cluster_flavor_name,
-                                            cluster_volume, "master", None),
+                                            cluster_volume, "master", None,
+                                            None),
             cluster_manager.ClusterInstance("id2", "name2", cluster_flavor,
                                             cluster_flavor_name,
-                                            cluster_volume, "slave", "master"),
+                                            cluster_volume, "slave",
+                                            "master", None),
             cluster_manager.ClusterInstance("id3", None, cluster_flavor,
                                             cluster_flavor_name,
-                                            cluster_volume, None, None),
+                                            cluster_volume, None, None, None),
         ]
 
         manager = cluster_manager.ClusterInstanceManager(cluster.id)
@@ -603,3 +657,10 @@ class ClustersTests(test.TestCase):
             if datastore in key:
                 return True
         return False
+
+    def _build_datastore_display_text(self, datastore, datastore_version):
+        return datastore + ' - ' + datastore_version
+
+    def _build_flavor_widget_name(self, datastore, datastore_version):
+        return binascii.hexlify(self._build_datastore_display_text(
+            datastore, datastore_version))
