@@ -55,10 +55,6 @@ class DeleteInstance(tables.DeleteAction):
             count
         )
 
-    name = "delete"
-    classes = ("btn-danger", )
-    icon = "remove"
-
     def delete(self, request, obj_id):
         api.trove.instance_delete(request, obj_id)
 
@@ -692,7 +688,8 @@ class UsersTable(tables.DataTable):
         row_actions = [EditUser, ManageAccess, DeleteUser]
 
     def get_object_id(self, datum):
-        return datum.name
+        obj_id = datum.name + "@" + datum.host
+        return obj_id
 
 
 class DatabaseTable(tables.DataTable):
