@@ -597,7 +597,7 @@ class DatabaseTests(test.TestCase):
         table.maybe_handle()
 
         self.assertEqual(table.data[0].enabled, True)
-        self.assertEqual(table.data[0].password, None)
+        self.assertIsNone(table.data[0].password)
 
     @test.create_stubs({api.trove: ('root_disable',)})
     def test_disable_root_exception(self):
@@ -635,7 +635,7 @@ class DatabaseTests(test.TestCase):
 
         database_id = database.id
         # Instead of using the user's ID, the api uses the user's name. BOOO!
-        user_id = user.name
+        user_id = user.name + "@" + user.host
 
         # views.py: DetailView.get_data
         api.trove.instance_get(IsA(http.HttpRequest), IsA(six.text_type))\
