@@ -16,7 +16,6 @@
 Views for managing database instances.
 """
 from collections import OrderedDict
-import logging
 
 from django.core.urlresolvers import reverse
 from django.core.urlresolvers import reverse_lazy
@@ -32,6 +31,7 @@ from horizon.utils import memoized
 from horizon import workflows as horizon_workflows
 from openstack_dashboard.dashboards.project.instances \
     import utils as instance_utils
+from oslo_log import log as logging
 
 from trove_dashboard import api
 from trove_dashboard.content.databases import forms
@@ -163,7 +163,7 @@ class EditUserView(horizon_forms.ModalFormView):
 class AccessDetailView(horizon_tables.DataTableView):
     table_class = tables.AccessTable
     template_name = 'project/databases/access_detail.html'
-    page_title = _("Database Access for: {{ user_name }}")
+    page_title = _("Database Access for: {{ user_name }}@{{user_host}}")
 
     @memoized.memoized_method
     def get_data(self):
