@@ -229,6 +229,11 @@ BACKUP_ONE = {
     "size": 0.13,
     "id": "0edb3c14-8919-4583-9add-00df9e524081",
     "description": "Long description of backup",
+    "datastore": {
+        "type": "mysql",
+        "version": "5.6",
+        "version_id": "500a6d52-8347-4e00-8e4c-f4fa9cf96ae9"
+    },
 }
 
 BACKUP_TWO = {
@@ -241,6 +246,11 @@ BACKUP_TWO = {
     "size": 0.13,
     "id": "e4602a3c-2bca-478f-b059-b6c215510fb4",
     "description": "Longer description of backup",
+    "datastore": {
+        "type": "mysql",
+        "version": "5.6",
+        "version_id": "500a6d52-8347-4e00-8e4c-f4fa9cf96ae9"
+    },
 }
 
 BACKUP_TWO_INC = {
@@ -254,6 +264,11 @@ BACKUP_TWO_INC = {
     "id": "e4602a3c-2bca-478f-b059-b6c215510fb5",
     "description": "Longer description of backup",
     "parent_id": "e4602a3c-2bca-478f-b059-b6c215510fb4",
+    "datastore": {
+        "type": "mysql",
+        "version": "5.6",
+        "version_id": "500a6d52-8347-4e00-8e4c-f4fa9cf96ae9"
+    },
 }
 
 CONFIG_ONE = {
